package cc.smtweb.system.bpm.web.engine.dynPage;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwException;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityHelper;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.mvc.service.SqlNamedPara;
import cc.smtweb.framework.core.mvc.service.SwListData;
import cc.smtweb.system.bpm.web.design.form.define.PageDataset;

import java.util.List;

import static cc.smtweb.framework.core.common.SwConsts.TOTAL_KEY;

/**
 * Created by Akmm at 2022/4/21 17:53
 */
public class DynPageLoadOneHandler extends AbstractDynPageHandler {
    //数据集
    private String dbName;
    //过滤条件
    private SwMap filter = new SwMap();

    //对应的数据集定义
    private PageDataset pageDataSet;

    public R load() throws Exception {
        dbName = params.readString("dataset");
        filter = (SwMap)params.get("filter");

        pageDataSet = findDataset(dbName);
        if (pageDataSet == null) throw new SwException("没有找到指定的的数据集定义：" + dbName + "！");

        DynRetBean bean = null;
        if (SwEnum.DatasetType.LIST.value.equals(pageDataSet.type)) {//列表类
            bean = DynRetBean.createList(loadList());
        } else if (SwEnum.DatasetType.TREE.value.equals(pageDataSet.type)) {//列表类
//            return new DynRetBean(loadTree());
        } else {//列表类
            bean = DynRetBean.createBean(loadOne());
        }

        return R.success(bean);
    }

    /**
     * 返回单个对象
     * @return
     */
    private SwMap loadOne() {
        SqlNamedPara sqlPara = DynPageHelper.buildSelectSql(pageDataSet, filter);
        SwMap map = DbEngine.getInstance().queryEntityN(sqlPara.sql, sqlPara.paras, SwMap.class);
        if (map == null) {
            throw new SwException("没有找到指定数据（ds=" + pageDataSet.name + "）");
        }
        ModelTable masterTable = ModelTableCache.getInstance().get(pageDataSet.masterTable);
        EntityHelper.loadBeanLink(masterTable.getName(), map, sqlPara.mapFieldAlias);
        return map;
    }

    /**
     * 返回list
     * @return
     */
    private SwListData loadList() {
        SqlNamedPara sqlPara = DynPageHelper.buildSelectSql(pageDataSet, filter);
        List<SwMap> list;
        if (sqlPara.page > 0 && sqlPara.rows > 0) {
            list = DbEngine.getInstance().pagedQueryN(sqlPara.sql, SwMap.class, (sqlPara.page - 1) * sqlPara.rows, sqlPara.rows, sqlPara.paras);
        } else {
            list = DbEngine.getInstance().queryN(sqlPara.sql, sqlPara.paras, SwMap.class);
        }
        ModelTable masterTable = ModelTableCache.getInstance().get(pageDataSet.masterTable);
        EntityHelper.loadBeanLink(masterTable.getName(), list, sqlPara.mapFieldAlias);
        return SwListData.create(list, sqlPara.rows);
    }

    /**
     * 计算分页数据
     * @return
     */
    public R getTotal() {
        try {
            SqlNamedPara sqlPara = DynPageHelper.buildSumSql(pageDataSet, filter);

            SwMap mapFooter = DbEngine.getInstance().queryEntityN(sqlPara.sql, sqlPara.paras, SwMap.class);

            R r = R.success();
            r.put("total", mapFooter.get(TOTAL_KEY));
            r.put("footer", mapFooter);
            return r;
        } catch (Exception e) {
            return R.error("计算合计失败！", e);
        }
    }
}
